# coding=utf8
# -*- coding: utf8 -*-
# vim: set fileencoding=utf8 :

from __future__ import unicode_literals
from django.conf import settings
from rest_framework import VERSION, serializers
from rest_framework.response import Response
from rest_messaging.pagination import MessagePagination


DRFVLIST = [int(x) for x in VERSION.split(".")]


def compat_serializer_method_field(method_name=None):
    """ method_name changed in DRF > 3. See http://www.django-rest-framework.org/topics/3.0-announcement/#optional-argument-to-serializermethodfield. """
    if DRFVLIST[0] >= 3:
        return serializers.SerializerMethodField()
    else:
        return serializers.SerializerMethodField(method_name=method_name)


def compat_serializer_check_is_valid(serializer):
    """ http://www.django-rest-framework.org/topics/3.0-announcement/#using-is_validraise_exceptiontrue """
    if DRFVLIST[0] >= 3:
        serializer.is_valid(raise_exception=True)
    else:
        if not serializer.is_valid():
            serializers.ValidationError('The serializer raises a validation error')


def compat_thread_serializer_set():
    """ We create the Thread manually and must assign it to the serializer. DRF 3 uses serializer.instance while DRF 2 uses serializer.object """
    if DRFVLIST[0] >= 3:
        return "instance"
    else:
        return "object"


def compat_serializer_attr(serializer, obj):
    """
    Required only for DRF 3.1, which does not make dynamically added attribute available in obj in serializer.
    This is a quick solution but works without breajing anything.
    """
    if DRFVLIST[0] == 3 and DRFVLIST[1] == 1:
        for i in serializer.instance:
            if i.id == obj.id:
                return i
    else:
        return obj


def compat_get_request_data(request):
    """ http://www.django-rest-framework.org/topics/3.0-announcement/#request-objects """
    if DRFVLIST[0] >= 3:
        return request.data
    else:
        return request.DATA


def compat_perform_update(instance, serializer):
    """ Verbatim copy of compat_perform_update mixin for DRF 2.4 compatibility. """
    if DRFVLIST[0] == 2:
        serializer.save()


def compat_get_paginated_response(view, page):
    """ get_paginated_response is unknown to DRF 3.0 """
    if DRFVLIST[0] == 3 and DRFVLIST[1] >= 1:
        from rest_messaging.serializers import ComplexMessageSerializer  # circular import
        serializer = ComplexMessageSerializer(page, many=True)
        return view.get_paginated_response(serializer.data)
    else:
        serializer = view.get_pagination_serializer(page)
        return Response(serializer.data)


def compat_pagination_messages(cls):
    """
    For DRF 3.1 and higher, pagination is defined at the paginator level (see http://www.django-rest-framework.org/topics/3.2-announcement/).
    For DRF 3.0 and lower, it can be handled at the view level.
    """
    if DRFVLIST[0] == 3 and DRFVLIST[1] >= 1:
        setattr(cls, "pagination_class", MessagePagination)
        return cls
    else:
        # DRF 2 pagination
        setattr(cls, "paginate_by", getattr(settings, "DJANGO_REST_MESSAGING_MESSAGES_PAGE_SIZE", 30))
        return cls



# coding=utf8
# -*- coding: utf8 -*-
# vim: set fileencoding=utf8 :

from __future__ import unicode_literals
from rest_framework import permissions


class IsInThread(permissions.BasePermission):
    """
    Custom permission to only allow participants to a thread to access it.
    """
    def has_object_permission(self, request, view, obj):
        """ Here we ensure the user is in the thread. """
        return obj.is_participant(request)

    def has_permission(self, request, view):
        """ We always required a logged in user. """
        if request.user.is_authenticated():
            return True
        return False



# coding=utf8
# -*- coding: utf8 -*-
# vim: set fileencoding=utf8 :

from __future__ import unicode_literals
from django.conf.urls import include, url
from rest_framework import routers
from rest_messaging import views


# Create a router and register our viewsets with it.
router = routers.SimpleRouter()
router.register(r'threads', views.ThreadView, 'threads')
router.register(r'messages', views.MessageView, 'messages')
router.register(r'notifications', views.NotificationCheckView, 'notifications')
router.register(r'authentication', views.ParticipantAuthenticationView, 'authentication')

# API endpoints, determined automatically by the router.
urlpatterns = [
    url(r'^', include(router.urls)),
]



# coding=utf8
# -*- coding: utf8 -*-
# vim: set fileencoding=utf8 :

from __future__ import unicode_literals
from django.conf import settings
from django.db import models
from django.db.models import Count, Max
from django.db.models.signals import post_save
from django.utils.encoding import python_2_unicode_compatible
from django.utils.timezone import now, timedelta


@python_2_unicode_compatible
class Participant(models.Model):
    """
    The participant model holds a django.contrib.auth.models.User's id.
    This allows us to use rest_messaging without querying the main db.
    """
    id = models.PositiveIntegerField(primary_key=True)

    def __str__(self):
        return "{0}".format(self.id)


class ThreadManager(models.Manager):

    def get_threads_for_participant(self, participant_id):
        """ Gets all the threads in which the current participant is or was involved. The method does not exclude threads where the participant has left. """
        return Thread.objects.\
            filter(participants__id=participant_id).\
            distinct()

    def get_threads_where_participant_is_active(self, participant_id):
        """ Gets all the threads in which the current participant is involved. The method excludes threads where the participant has left. """
        participations = Participation.objects.\
            filter(participant__id=participant_id).\
            exclude(date_left__lte=now()).\
            distinct().\
            select_related('thread')

        return Thread.objects.\
            filter(id__in=[p.thread.id for p in participations]).\
            distinct()

    def get_active_threads_involving_all_participants(self, *participant_ids):
        """ Gets the threads where the specified participants are active and no one has left. """

        query = Thread.objects.\
            exclude(participation__date_left__lte=now()).\
            annotate(count_participants=Count('participants')).\
            filter(count_participants=len(participant_ids))

        for participant_id in participant_ids:
            query = query.filter(participants__id=participant_id)

        return query.distinct()

    def get_or_create_thread(self, request, name=None, *participant_ids):
        """
        When a Participant posts a message to other participants without specifying an existing Thread,
        we must
        1. Create a new Thread if they have not yet opened the discussion.
        2. If they have already opened the discussion and multiple Threads are not allowed for the same users, we must
            re-attach this message to the existing thread.
        3. If they have already opened the discussion and multiple Threads are allowed, we simply create a new one.
        """

        # we get the current participant
        # or create him if he does not exit

        participant_ids = list(participant_ids)
        if request.rest_messaging_participant.id not in participant_ids:
            participant_ids.append(request.rest_messaging_participant.id)

        # we need at least one other participant
        if len(participant_ids) < 2:
            raise Exception('At least two participants are required.')

        if getattr(settings, "REST_MESSAGING_THREAD_UNIQUE_FOR_ACTIVE_RECIPIENTS", True) is True:
            # if we limit the number of threads by active participants
            # we ensure a thread is not already running
            existing_threads = self.get_active_threads_involving_all_participants(*participant_ids)
            if len(list(existing_threads)) > 0:
                return existing_threads[0]

        # we have no existing Thread or multiple Thread instances are allowed
        thread = Thread.objects.create(name=name)

        # we add the participants
        thread.add_participants(request, *participant_ids)

        # we send a signal to say the thread with participants is created
        post_save.send(Thread, instance=thread, created=True, created_and_add_participants=True, request_participant_id=request.rest_messaging_participant.id)

        return thread


@python_2_unicode_compatible
class Thread(models.Model):
    """
    A Thread groups messages using their recipients.
    """
    name = models.CharField(max_length=255, null=True, blank=True)
    participants = models.ManyToManyField(Participant, through='Participation')
    objects = models.Manager()
    managers = ThreadManager()

    def __str__(self):
        return self.name if self.name else "Thread {0}".format(self.id)

    def is_participant(self, request, *args, **kwargs):
        """ We ensure request.user is a participant to the thread. """
        participants = self.participants.all()
        try:
            current = Participant.objects.get(id=request.user.id)
            if current in participants:
                return current
        except:
            pass
        return False

    def add_participants(self, request, *participants_ids):
        """
        Ensures the current user has the authorization to add the participants.
        By default, a user can add a participant if he himself is a participant.
        A callback can be added in the settings here.
        """
        participants_ids_returned_by_callback = getattr(settings, 'REST_MESSAGING_ADD_PARTICIPANTS_CALLBACK', self._limit_participants)(request, *participants_ids)
        participations = []
        ids = []
        for participant_id in participants_ids_returned_by_callback:
            participations.append(Participation(participant_id=participant_id, thread=self))
            ids.append(participant_id)

        Participation.objects.bulk_create(participations)
        post_save.send(Thread, instance=self, created=True, created_and_add_participants=True, request_participant_id=request.rest_messaging_participant.id)

        return ids

    def _limit_participants(self, request, *participants_ids):
        """ By default, we ensure we do not have more than 10 participants. """
        participants_all = self.participants.all().values_list('id', flat=True)

        max = 10 - len(participants_all)
        lst = []
        for index, participant_id in enumerate(participants_ids):
            if index >= max:
                break
            if participant_id not in participants_all:
                lst.append(participant_id)
        return lst

    def remove_participant(self, request, participant):
        removable_participants_ids = self.get_removable_participants_ids(request)
        if participant.id in removable_participants_ids:
            participation = Participation.objects.get(participant=participant, thread=self, date_left=None)
            participation.date_left = now()
            participation.save()
            post_save.send(Thread, instance=self, created=False, remove_participant=True, removed_participant=participant, request_participant_id=request.rest_messaging_participant.id)
            return participation
        else:
            raise Exception('The participant may not be removed.')

    def get_removable_participants_ids(self, request):
        removable_participants_ids = getattr(settings, 'REST_MESSAGING_REMOVE_PARTICIPANTS_CALLBACK', self._can_remove_oneself_only)(request, self)
        return removable_participants_ids

    def _can_remove_oneself_only(self, request, participant):
        """ By default, we ensure request.user can only remove oneself. """
        try:
            return [self.is_participant(request).id]
        except:
            return []


class Participation(models.Model):
    """
    Links Participant to threads.
    Tells us when a participant has joined a Thread and when he left.
    """
    participant = models.ForeignKey(Participant)
    thread = models.ForeignKey(Thread)
    date_joined = models.DateTimeField(auto_now_add=True)
    date_left = models.DateTimeField(null=True, blank=True)
    date_last_check = models.DateTimeField(null=True, blank=True)  # a timestamp to be set when a participant reads a thread


class MessageManager(models.Manager):

    def return_daily_messages_count(self, sender):
        """ Returns the number of messages sent in the last 24 hours so we can ensure the user does not exceed his messaging limits """
        h24 = now() - timedelta(days=1)
        return Message.objects.filter(sender=sender, sent_at__gte=h24).count()

    def check_who_read(self, messages):
        """ Check who read each message. """
        # we get the corresponding Participation objects
        for m in messages:
            readers = []
            for p in m.thread.participation_set.all():
                if p.date_last_check is None:
                    pass
                elif p.date_last_check > m.sent_at:
                    # the message has been read
                    readers.append(p.participant.id)
            setattr(m, "readers", readers)

        return messages

    def check_is_notification(self, participant_id, messages):
        """ Check if each message requires a notification for the specified participant. """
        try:
            # we get the last check
            last_check = NotificationCheck.objects.filter(participant__id=participant_id).latest('id').date_check
        except Exception:
            # we have no notification check
            # all the messages are considered as new
            for m in messages:
                m.is_notification = True
            return messages

        for m in messages:
            if m.sent_at > last_check and m.sender.id != participant_id:
                setattr(m, "is_notification", True)
            else:
                setattr(m, "is_notification", False)
        return messages

    def get_lasts_messages_of_threads(self, participant_id, check_who_read=True, check_is_notification=True):
        """ Returns the last message in each thread """
        # we get the last message for each thread
        # we must query the messages using two queries because only Postgres supports .order_by('thread', '-sent_at').distinct('thread')
        threads = Thread.managers.\
            get_threads_where_participant_is_active(participant_id).\
            annotate(last_message_id=Max('message__id'))
        messages = Message.objects.filter(id__in=[thread.last_message_id for thread in threads]).\
            order_by('-id').\
            distinct().\
            select_related('thread', 'sender')

        if check_who_read is True:
            messages = messages.prefetch_related('thread__participation_set', 'thread__participation_set__participant')
            messages = self.check_who_read(messages)
        else:
            messages = messages.prefetch_related('thread__participants')

        if check_is_notification is True:
            messages = self.check_is_notification(participant_id, messages)
        return messages

    def get_all_messages_in_thread(self, participant_id, thread_id, check_who_read=True):
        """ Returns all the messages in a thread. """
        try:
            messages = Message.objects.filter(thread__id=thread_id).\
                order_by('-id').\
                select_related('thread').\
                prefetch_related('thread__participation_set', 'thread__participation_set__participant')
        except Exception:
            return Message.objects.none()

        messages = self.check_who_read(messages)
        return messages


@python_2_unicode_compatible
class Message(models.Model):
    """
    A message between a User and another User or an AnonymousUser.
    """

    body = models.TextField(null=False)
    sender = models.ForeignKey(Participant, null=False)
    thread = models.ForeignKey(Thread)
    sent_at = models.DateTimeField(auto_now_add=True, blank=True)
    objects = models.Manager()
    managers = MessageManager()

    def __str__(self):
        return "{0}: {1}".format(self.sender, "{0}".format(self.body[:15]))

    def save(self, *args, **kwargs):
        """ Checks if there is a daily limit to the number of messages that can be sent. """
        max_messages = getattr(settings, 'REST_MESSAGING_DAILY_LIMIT_CALLBACK', lambda message_instance, *args, **kwargs: None)(self, *args, **kwargs)
        if max_messages is None or Message.managers.return_daily_messages_count(self.sender) < max_messages:
            super(Message, self).save(*args, **kwargs)
        else:
            # participant cannot write anymore today
            raise Exception('The daily messaging limit has been reached for this sender')


@python_2_unicode_compatible
class NotificationCheck(models.Model):
    """
    A timestamp everytime a user checks his notifications
    """
    participant = models.OneToOneField(Participant)
    date_check = models.DateTimeField()

    def __str__(self):
        return "{0}: {1}".format(self.participant, self.date_check)



# coding=utf8
# -*- coding: utf8 -*-
# vim: set fileencoding=utf8 :

from __future__ import unicode_literals

from django.conf import settings
from django.utils.timezone import now
from rest_framework import mixins, permissions, status, viewsets
from rest_framework.decorators import detail_route, list_route
from rest_framework.response import Response
from rest_messaging.compat import compat_get_paginated_response, compat_get_request_data, compat_pagination_messages, compat_serializer_check_is_valid, compat_thread_serializer_set, compat_perform_update
from rest_messaging.models import Message, NotificationCheck, Participant, Participation, Thread
from rest_messaging.permissions import IsInThread
from rest_messaging.serializers import MessageNotificationCheckSerializer, ComplexMessageSerializer, SimpleMessageSerializer, ThreadSerializer
import json


class ThreadView(mixins.RetrieveModelMixin,
                 mixins.CreateModelMixin,
                 mixins.UpdateModelMixin,
                 viewsets.GenericViewSet):
    """
    The ThreadView allow us to create threads, and add/remove people to/from them.
    It does not list the messages belonging to the thread.
    """
    queryset = Thread.objects.all().prefetch_related('participants')
    serializer_class = ThreadSerializer
    permission_classes = (IsInThread,)

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = ThreadSerializer(instance, callback=getattr(settings, 'REST_MESSAGING_SERIALIZE_PARTICIPANTS_CALLBACK', None), context={'request': request})  # self.get_serializer will raise an error in DRF 2.4
        return Response(serializer.data)

    def create(self, request, *args, **kwargs):
        """ We ensure the Thread only involves eligible participants. """
        serializer = self.get_serializer(data=compat_get_request_data(request))
        compat_serializer_check_is_valid(serializer)
        self.perform_create(request, serializer)
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    def perform_create(self, request, serializer):
        participants_ids = json.loads(compat_get_request_data(self.request).get('participants'))
        thread = Thread.managers.get_or_create_thread(self.request, compat_get_request_data(self.request).get('name'), *participants_ids)
        setattr(serializer, compat_thread_serializer_set(), thread)

    def update(self, request, *args, **kwargs):
        participants_ids = compat_get_request_data(self.request).getlist('participants', [])
        if len(participants_ids) > 0:
            # we warn the user he cannot update the participants here
            return Response("Participant updates not allowed by this method.", status=status.HTTP_400_BAD_REQUEST)
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=compat_get_request_data(request), partial=partial)
        compat_serializer_check_is_valid(serializer)
        try:
            self.perform_update(serializer)
        except:
            compat_perform_update(self, serializer)
        return Response(serializer.data)

    @detail_route(methods=['post'])
    def add_participants(self, request, pk=None):
        # we get the thread and check for permission
        thread = Thread.objects.get(id=pk)
        self.check_object_permissions(request, thread)
        # we get the participants and add them
        participants_ids = json.loads(compat_get_request_data(self.request).get('participants'))
        thread.add_participants(request, *participants_ids)
        # we return the thread
        serializer = self.get_serializer(thread)
        return Response(serializer.data)

    @detail_route(methods=['post'])
    def remove_participant(self, request, pk=None):
        # we get the thread and check for permission
        thread = Thread.objects.get(id=pk)
        self.check_object_permissions(request, thread)
        # we get the participant
        participant_id = compat_get_request_data(self.request).get('participant')
        participant = Participant.objects.get(id=participant_id)
        # we remove him if thread.remove_participant allows us to
        try:
            thread.remove_participant(request, participant)
            # we return the thread
            serializer = self.get_serializer(thread)
            return Response(serializer.data)
        except Exception:
            return Response(status=status.HTTP_400_BAD_REQUEST)

    @detail_route(methods=['get'])
    def get_removable_participants_ids(self, request, pk=None):
        # we get the thread and check for permission
        thread = Thread.objects.get(id=pk)
        self.check_object_permissions(request, thread)
        # we get the removable participants
        removable_participants_ids = thread.get_removable_participants_ids(request)
        # we remove him if thread.remove_participant allows us to
        try:
            return Response({'participants': removable_participants_ids})
        except Exception:
            return Response(status=status.HTTP_400_BAD_REQUEST)

    @detail_route(methods=['post'])
    def mark_thread_as_read(self, request, pk=None):
        """ Pk is the pk of the Thread to which the messages belong. """
        # we get the thread and check for permission
        thread = Thread.objects.get(id=pk)
        self.check_object_permissions(request, thread)
        # we save the date
        try:
            participation = Participation.objects.get(thread=thread, participant=request.rest_messaging_participant)
            participation.date_last_check = now()
            participation.save()
            # we return the thread
            serializer = self.get_serializer(thread)
            return Response(serializer.data)
        except Exception:
            return Response(status=status.HTTP_400_BAD_REQUEST)


@compat_pagination_messages
class MessageView(mixins.ListModelMixin,
                  viewsets.GenericViewSet):
    """ The view only lists and creates. """

    queryset = Message.objects.none()
    serializer_class = ComplexMessageSerializer
    permission_classes = (IsInThread,)

    def get_queryset(self):
        """ We list all the threads involving the user """
        check_notifications = self.request.GET.get("check_notifications", True)
        messages = Message.managers.get_lasts_messages_of_threads(self.request.rest_messaging_participant.id, check_who_read=True, check_is_notification=check_notifications)
        return messages

    @detail_route(methods=['post'], permission_classes=[IsInThread], serializer_class=SimpleMessageSerializer)
    def post_message(self, request, pk=None):
        """ Pk is the pk of the Thread to which the message belongs. """
        # we get the thread and check for permission
        thread = Thread.objects.get(id=pk)
        self.check_object_permissions(request, thread)
        # we get the body
        body = compat_get_request_data(self.request).get('body')
        # we create the message
        # Message.objects.save() could return an Exception
        try:
            message = Message(sender=request.rest_messaging_participant, thread=thread, body=body)
            message.save()
            serializer = SimpleMessageSerializer(message)
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except Exception:
            return Response(status=status.HTTP_412_PRECONDITION_FAILED)

    @detail_route(methods=['get'], permission_classes=[IsInThread], serializer_class=ComplexMessageSerializer)
    def list_messages_in_thread(self, request, pk=None):
        """ Pk is the pk of the Thread to which the messages belong. """
        # we get the thread and check for permission
        thread = Thread.objects.get(id=pk)
        self.check_object_permissions(request, thread)
        messages = Message.managers.get_all_messages_in_thread(participant_id=request.rest_messaging_participant.id, thread_id=thread.id, check_who_read=True)
        page = self.paginate_queryset(messages)
        if page is not None:
            return compat_get_paginated_response(self, page)
        serializer = ComplexMessageSerializer(messages, many=True)
        return Response(serializer.data)


class NotificationCheckView(mixins.ListModelMixin, viewsets.GenericViewSet):

    serializer_class = MessageNotificationCheckSerializer
    permission_classes = (permissions.IsAuthenticated,)

    @list_route(methods=['post'])
    def check(self, request, *args, **kwargs):
        # we get the NotificationCheck instance corresponding to the user or we create it

        try:
            nc = NotificationCheck.objects.get(participant=request.rest_messaging_participant)
            if nc:
                nc.date_check = now()
                nc.save()
            status_code = status.HTTP_200_OK

        except Exception:
            nc = NotificationCheck.objects.create(participant=request.rest_messaging_participant, date_check=now())
            status_code = status.HTTP_201_CREATED

        serializer = self.get_serializer(nc)
        return Response(serializer.data, status=status_code)


class ParticipantAuthenticationView(mixins.ListModelMixin, viewsets.GenericViewSet):
    """
    View that simply return the participant id of the user as set by the middleware, if it exists.
    """

    permission_classes = (permissions.IsAuthenticated,)

    def list(self, request, *args, **kwargs):
        participant = Participant.objects.get(id=self.request.rest_messaging_participant.id)
        return Response({'id': participant.id})



# coding=utf8
# -*- coding: utf8 -*-
# vim: set fileencoding=utf8 :

from __future__ import unicode_literals
from django.core.cache import cache
from rest_messaging.models import Participant


class MessagingMiddleware(object):
    """
    Ensures we can access request.user as request.rest_messaging_participant in every request.
    """

    def process_view(self, request, callback, callback_args, callback_kwargs):

        assert hasattr(request, 'user'), (
            "The django-rest-messaging MessagingMiddleware requires an authentication middleware "
            "to be installed because request.user must be available."
        )

        if request.user.is_authenticated():

            participant = cache.get('rest_messaging_participant_{0}'.format(request.user.id), None)

            if participant is None:
                # either we create the participant or we retrieve him
                try:
                    participant = Participant.objects.get(id=request.user.id)
                except:
                    participant = Participant.objects.create(id=request.user.id)
                cache.set('rest_messaging_participant_{0}'.format(request.user.id), participant, 60 * 60)  # cached for 60 minutes
        else:
            participant = None

        request.rest_messaging_participant = participant

        return None



# coding=utf8
# -*- coding: utf8 -*-
# vim: set fileencoding=utf8 :

from __future__ import unicode_literals
from django.conf import settings

try:
    from rest_framework.pagination import PageNumberPagination
except:
    PageNumberPagination = object  # dummy import


class MessagePagination(PageNumberPagination):
    page_size = getattr(settings, "DJANGO_REST_MESSAGING_MESSAGES_PAGE_SIZE", 30)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('body', models.TextField()),
                ('sent_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='NotificationCheck',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date_check', models.DateTimeField()),
            ],
        ),
        migrations.CreateModel(
            name='Participant',
            fields=[
                ('id', models.PositiveIntegerField(serialize=False, primary_key=True)),
            ],
        ),
        migrations.CreateModel(
            name='Participation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date_joined', models.DateTimeField(auto_now_add=True)),
                ('date_left', models.DateTimeField(null=True, blank=True)),
                ('date_last_check', models.DateTimeField(null=True, blank=True)),
                ('participant', models.ForeignKey(to='rest_messaging.Participant')),
            ],
        ),
        migrations.CreateModel(
            name='Thread',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=255, null=True, blank=True)),
                ('participants', models.ManyToManyField(to='rest_messaging.Participant', through='rest_messaging.Participation')),
            ],
        ),
        migrations.AddField(
            model_name='participation',
            name='thread',
            field=models.ForeignKey(to='rest_messaging.Thread'),
        ),
        migrations.AddField(
            model_name='notificationcheck',
            name='participant',
            field=models.OneToOneField(to='rest_messaging.Participant'),
        ),
        migrations.AddField(
            model_name='message',
            name='sender',
            field=models.ForeignKey(to='rest_messaging.Participant'),
        ),
        migrations.AddField(
            model_name='message',
            name='thread',
            field=models.ForeignKey(to='rest_messaging.Thread'),
        ),
    ]



# coding=utf8
# -*- coding: utf8 -*-
# vim: set fileencoding=utf8 :

from __future__ import unicode_literals
from rest_framework import serializers
from rest_messaging.compat import compat_serializer_attr, compat_serializer_method_field
from rest_messaging.models import Message, NotificationCheck, Thread


class ThreadSerializer(serializers.ModelSerializer):

    participants = compat_serializer_method_field("get_participants")
    removable_participants_ids = compat_serializer_method_field("get_removable_participants_ids")

    class Meta:
        model = Thread
        fields = ('id', 'name', 'participants', 'removable_participants_ids')

    def __init__(self, *args, **kwargs):
        # Don't pass the 'callback' arg up to the superclass
        self.callback = kwargs.pop('callback', None)
        # Instantiate the superclass normally
        super(ThreadSerializer, self).__init__(*args, **kwargs)

    def get_participants(self, obj):
        """ Allows to define a callback for serializing information about the user. """
        # we set the many to many serialization to False, because we only want it with retrieve requests
        if self.callback is None:
            return [participant.id for participant in obj.participants.all()]
        else:
            # we do not want user information
            return self.callback(obj)

    def get_removable_participants_ids(self, obj):
        """ Get the participants that can be removed from the thread. """
        return obj.get_removable_participants_ids(self.context.get('request', None))


class SimpleMessageSerializer(serializers.ModelSerializer):
    """ Returns the messages without complementary information. """

    class Meta:
        model = Message
        fields = ('id', 'body', 'sender', 'thread', 'sent_at')


class ComplexMessageSerializer(serializers.ModelSerializer):

    is_notification = compat_serializer_method_field("get_is_notification")
    readers = compat_serializer_method_field("get_readers")

    class Meta:
        model = Message
        fields = ('id', 'body', 'sender', 'thread', 'sent_at', 'is_notification', 'readers')

    def get_is_notification(self, obj):
        """ We say if the message should trigger a notification """
        try:
            o = compat_serializer_attr(self, obj)
            return o.is_notification
        except Exception:
            return False

    def get_readers(self, obj):
        """ Return the ids of the people who read the message instance. """
        try:
            o = compat_serializer_attr(self, obj)
            return o.readers
        except Exception:
            return []


class MessageNotificationCheckSerializer(serializers.ModelSerializer):

    class Meta:
        model = NotificationCheck



# coding=utf8
# -*- coding: utf8 -*-
# vim: set fileencoding=utf8 :

from __future__ import unicode_literals
from django.apps import AppConfig


class MessagesAppConfig(AppConfig):

    name = 'rest_messaging'
    verbose_name = 'Messages App'

    def ready(self):
        # import signal handlers
        # from rest_messaging import signals
        pass  # we use no signal for now
